/// <reference types="react" />
import { Dimensions, Appearance, ScaledSize, EmitterSubscription, NativeEventSubscription, TextStyle, ImageStyle, ViewStyle, Platform, StyleProp } from "react-native";
import { MatchChildAtRuleOptions } from "./match-at-rule";
import { AtRuleTuple, MediaRecord } from "../types/common";
import { ColorSchemeStore, ColorSchemeSystem } from "./color-scheme";
export type { ColorSchemeSystem, ColorSchemeName } from "./color-scheme";
export declare type Style = ViewStyle | ImageStyle | TextStyle;
export declare type InlineStyle<T extends Style> = T;
export declare type AtRuleStyle<T extends Style> = T & {
    atRules: unknown[];
};
export declare type CompiledStyle = {
    [key: string]: string;
} & {
    $$css: boolean;
};
export declare type EitherStyle<T extends Style = Style> = AtRuleStyle<T> | CompiledStyle | InlineStyle<T> | StyleProp<T>;
export declare type Snapshot = Record<string, StylesArray>;
export declare type MatchAtRule = (options: SelectorOptions) => boolean | undefined;
export interface ChildStyle {
    className: string;
    style: Style;
    atRules: AtRuleTuple[];
}
export interface StylesArray<T = Style> extends Array<EitherStyle<T>> {
    dynamic?: boolean;
    isForChildren?: boolean;
    childStyles?: ChildStyle[];
    topics?: Set<string>;
}
export interface SelectorOptions {
    hover?: boolean;
    active?: boolean;
    focus?: boolean;
    componentHover?: boolean;
    componentActive?: boolean;
    componentFocus?: boolean;
}
declare global {
    var tailwindcss_react_native_style: Record<string, Style>;
    var tailwindcss_react_native_media: MediaRecord;
}
interface StyleSheetStoreConstructor {
    styles?: typeof global.tailwindcss_react_native_style;
    atRules?: typeof global.tailwindcss_react_native_media;
    dimensions?: Dimensions;
    appearance?: typeof Appearance;
    platform?: typeof Platform.OS;
    preprocessed?: boolean;
    colorScheme?: ColorSchemeSystem;
}
/**
 * Tailwind styles are strings of atomic classes. eg "a b" compiles to [a, b]
 *
 * If the styles are static we can simply cache them and return a stable result
 *
 * However, if the styles are dynamic (have atRules) there are two things we need to do
 *  - Update the atomic style
 *  - Update the dependencies of the atomic style
 *
 * This is performed by each style subscribing to a atRule topic. The atomic styles are updated
 * before the parent styles.
 *
 * The advantage of this system is that styles are only updated once, no matter how many components
 * are on using them
 *
 * The disadvantages are
 * - Is that the store doesn't purge unused styles, so the listeners will continue to grow
 * - UI states (hover/active/focus) are considered separate styles
 *
 * If you are interested in helping me build a more robust store, please create an issue on Github.
 *
 */
export declare class StyleSheetStore extends ColorSchemeStore {
    snapshot: Snapshot;
    listeners: Set<() => void>;
    atRuleListeners: Set<(topics: string[]) => void>;
    dimensionListener: EmitterSubscription;
    appearanceListener: NativeEventSubscription;
    styles: Record<string, Style>;
    atRules: MediaRecord;
    preprocessed: boolean;
    platform: typeof Platform.OS;
    window: ScaledSize;
    orientation: OrientationLockType;
    constructor({ styles, atRules, dimensions, appearance, platform, preprocessed, colorScheme, }?: StyleSheetStoreConstructor);
    getSnapshot: () => Snapshot;
    subscribe: (listener: () => void) => () => boolean;
    destroy(): void;
    getServerSnapshot(): Snapshot;
    notify(): void;
    subscribeMedia(listener: (topics: string[]) => void): () => boolean;
    notifyMedia(topics: string[]): void;
    isEqual(a: StylesArray, b: StylesArray): boolean;
    getKey(className: string, { hover, active, focus, componentHover, componentActive, componentFocus, }?: SelectorOptions): string;
    createSelector(className: string, options?: SelectorOptions): (snapshot: Snapshot) => StylesArray;
    insertComposedStyle(className: string, key: string, options?: SelectorOptions): void;
    /**
     * ClassNames are made of multiple atomic styles. eg "a b" are the styles [a, b]
     *
     * This function will be called for each atomic style
     */
    upsertAtomicStyle(className: string, options?: SelectorOptions): StylesArray;
    getChildStyles<T>(parent: StylesArray<T>, options: MatchChildAtRuleOptions): StylesArray<Style> | undefined;
}
export declare const StoreContext: import("react").Context<StyleSheetStore>;
