"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.StoreContext = exports.StyleSheetStore = void 0;
const react_1 = require("react");
const react_native_1 = require("react-native");
const match_at_rule_1 = require("./match-at-rule");
const selector_1 = require("../shared/selector");
const vh_1 = __importDefault(require("./units/vh"));
const vw_1 = __importDefault(require("./units/vw"));
const color_scheme_1 = require("./color-scheme");
const unknownStyles = [];
(_a = globalThis.tailwindcss_react_native_style) !== null && _a !== void 0 ? _a : (globalThis.tailwindcss_react_native_style = {});
(_b = globalThis.tailwindcss_react_native_media) !== null && _b !== void 0 ? _b : (globalThis.tailwindcss_react_native_media = {});
const units = {
    vw: vw_1.default,
    vh: vh_1.default,
};
/**
 * Tailwind styles are strings of atomic classes. eg "a b" compiles to [a, b]
 *
 * If the styles are static we can simply cache them and return a stable result
 *
 * However, if the styles are dynamic (have atRules) there are two things we need to do
 *  - Update the atomic style
 *  - Update the dependencies of the atomic style
 *
 * This is performed by each style subscribing to a atRule topic. The atomic styles are updated
 * before the parent styles.
 *
 * The advantage of this system is that styles are only updated once, no matter how many components
 * are on using them
 *
 * The disadvantages are
 * - Is that the store doesn't purge unused styles, so the listeners will continue to grow
 * - UI states (hover/active/focus) are considered separate styles
 *
 * If you are interested in helping me build a more robust store, please create an issue on Github.
 *
 */
class StyleSheetStore extends color_scheme_1.ColorSchemeStore {
    constructor({ styles = global.tailwindcss_react_native_style, atRules = global.tailwindcss_react_native_media || [], dimensions = react_native_1.Dimensions, appearance = react_native_1.Appearance, platform = react_native_1.Platform.OS, preprocessed = false, colorScheme, } = {}) {
        super(colorScheme);
        this.snapshot = {};
        this.listeners = new Set();
        this.atRuleListeners = new Set();
        this.getSnapshot = () => {
            return this.snapshot;
        };
        this.subscribe = (listener) => {
            this.listeners.add(listener);
            return () => this.listeners.delete(listener);
        };
        this.platform = platform;
        this.styles = styles;
        this.atRules = atRules;
        this.preprocessed = preprocessed;
        this.window = dimensions.get("window");
        const screen = dimensions.get("screen");
        this.orientation = screen.height >= screen.width ? "portrait" : "landscape";
        this.dimensionListener = dimensions.addEventListener("change", ({ window, screen }) => {
            const topics = ["window"];
            if (window.width !== this.window.width)
                topics.push("width");
            if (window.height !== this.window.height)
                topics.push("height");
            this.window = window;
            const orientation = screen.height >= screen.width ? "portrait" : "landscape";
            if (orientation !== this.orientation)
                topics.push("orientation");
            this.orientation = orientation;
            this.notifyMedia(topics);
        });
        this.appearanceListener = appearance.addChangeListener(({ colorScheme }) => {
            if (this.colorSchemeSystem === "system") {
                this.colorScheme = colorScheme || "light";
                this.notifyMedia(["colorScheme"]);
            }
        });
    }
    destroy() {
        this.dimensionListener.remove();
        this.appearanceListener.remove();
    }
    getServerSnapshot() {
        return this.snapshot;
    }
    notify() {
        for (const l of this.listeners)
            l();
    }
    subscribeMedia(listener) {
        this.atRuleListeners.add(listener);
        return () => this.atRuleListeners.delete(listener);
    }
    notifyMedia(topics) {
        for (const l of this.atRuleListeners)
            l(topics);
        this.notify();
    }
    isEqual(a, b) {
        if (a.length !== b.length) {
            return false;
        }
        return a.every((style, index) => Object.is(style, b[index]));
    }
    getKey(className, { hover = false, active = false, focus = false, componentHover = false, componentActive = false, componentFocus = false, } = {}) {
        return this.preprocessed
            ? className
            : `${className}.${+hover}${+active}${+focus}${+componentHover}${+componentActive}${+componentFocus}`;
    }
    createSelector(className, options = {}) {
        if (typeof className !== "string") {
            return () => unknownStyles;
        }
        const key = this.getKey(className, options);
        if (!this.snapshot[key]) {
            this.insertComposedStyle(className, key, options);
        }
        return ((snapshot) => {
            return snapshot[key];
        }).bind(this);
    }
    insertComposedStyle(className, key, options = {}) {
        if (this.preprocessed) {
            const styleArray = [
                { $$css: true, [key]: key },
            ];
            styleArray.dynamic = false;
            this.snapshot = { ...this.snapshot, [key]: styleArray };
            return;
        }
        const topics = new Set();
        let init = true;
        let isDynamic = false;
        let childStyles;
        const reEvaluate = () => {
            const styleArray = [];
            for (const name of className.split(/\s+/)) {
                const classNameStyles = this.upsertAtomicStyle(name, options);
                styleArray.push(...classNameStyles);
                // These values will not change, so we can skip them after the first run
                if (init) {
                    if (classNameStyles.dynamic) {
                        isDynamic = true;
                    }
                    if (classNameStyles.childStyles) {
                        if (!childStyles) {
                            childStyles = [];
                        }
                        childStyles.push(...classNameStyles.childStyles);
                    }
                    for (const topic of classNameStyles.topics || []) {
                        topics.add(topic);
                    }
                }
            }
            styleArray.dynamic = isDynamic;
            styleArray.childStyles = childStyles;
            this.snapshot = { ...this.snapshot, [key]: styleArray };
        };
        reEvaluate();
        init = false;
        this.subscribeMedia((notificationTopics) => {
            if (notificationTopics.some((topic) => topics.has(topic))) {
                reEvaluate();
            }
        });
    }
    /**
     * ClassNames are made of multiple atomic styles. eg "a b" are the styles [a, b]
     *
     * This function will be called for each atomic style
     */
    upsertAtomicStyle(className, options = {}) {
        // This atomic style has already been processed, we can skip it
        if (this.snapshot[className])
            return this.snapshot[className];
        const normalizeClassName = (0, selector_1.normalizeSelector)(className);
        // To keep things consistent, even atomic styles are arrays
        const styleArray = this.styles[normalizeClassName]
            ? [this.styles[normalizeClassName]]
            : [];
        const atRulesTuple = this.atRules[normalizeClassName];
        // If there are no atRules, this style is static.
        // We can add it to the snapshot and early exit.
        if (!atRulesTuple) {
            styleArray.dynamic = false;
            this.snapshot = { ...this.snapshot, [className]: styleArray };
            return styleArray;
        }
        // Change the className to the dynamic version
        className = this.getKey(className, options);
        // The dynamic version may have already been processed, we can skip it
        if (this.snapshot[className])
            return this.snapshot[className];
        // These are the media topics the atomic style has subscribed to.
        // They may be things like window, window.width, orientation, colorScheme, etc
        const topics = new Set();
        // When a topic has new information, this function will be called.
        // Its purpose is to compute
        const reEvaluate = () => {
            const childStyles = [];
            const newStyles = [...styleArray];
            newStyles.dynamic = true;
            newStyles.topics = topics;
            for (const [index, atRules] of atRulesTuple.entries()) {
                let isForChildren = false;
                let unitKey;
                const childAtRules = [];
                const atRulesResult = atRules.every(([rule, params]) => {
                    /**
                     * This is a magic string, but it makes sense
                     * Child selectors look like this and will always start with (>
                     *
                     * @selector (> *:not(:first-child))
                     * @selector (> *)
                     */
                    if (rule === "selector" && params && params.startsWith("(>")) {
                        isForChildren = true;
                        childAtRules.push([rule, params]);
                        return true;
                    }
                    if (rule === "dynamic-style") {
                        if (unitKey) {
                            throw new Error("cannot have multiple unit keys");
                        }
                        unitKey = params;
                        return true;
                    }
                    return (0, match_at_rule_1.matchAtRule)({
                        rule,
                        params,
                        platform: this.platform,
                        width: this.window.width,
                        height: this.window.height,
                        colorScheme: this.colorScheme,
                        orientation: this.orientation,
                        ...options,
                    });
                });
                if (!atRulesResult) {
                    continue;
                }
                const stylesKey = `${normalizeClassName}.${index}`;
                const style = this.styles[stylesKey];
                if (unitKey) {
                    for (const [key, value] of Object.entries(style)) {
                        style[key] = units[unitKey](value);
                    }
                }
                if (isForChildren) {
                    childStyles.push({ className, style, atRules: childAtRules });
                }
                else {
                    newStyles.push(style);
                }
            }
            if (childStyles.length > 0) {
                newStyles.childStyles = childStyles;
            }
            const existingStyles = this.snapshot[className];
            if (!existingStyles) {
                this.snapshot[className] = newStyles;
                return newStyles;
            }
            if (this.isEqual(existingStyles, newStyles) &&
                this.isEqual(existingStyles.childStyles || [], newStyles.childStyles || [])) {
                return existingStyles;
            }
            this.snapshot = { ...this.snapshot, [className]: newStyles };
            return newStyles;
        };
        // Loop over the atRules and either subscribe to topics
        // or create matchAtRule functions
        for (const [[atRule, params]] of atRulesTuple) {
            if (atRule === "media" && params) {
                if (params.includes("width"))
                    topics.add("width");
                if (params.includes("height"))
                    topics.add("height");
                if (params.includes("orientation"))
                    topics.add("orientation");
                if (params.includes("aspect-ratio"))
                    topics.add("window");
                if (params.includes("prefers-color-scheme"))
                    topics.add("colorScheme");
            }
        }
        this.subscribeMedia((notificationTopics) => {
            if (notificationTopics.some((topic) => topics.has(topic))) {
                reEvaluate();
            }
        });
        return reEvaluate();
    }
    getChildStyles(parent, options) {
        if (!parent.childStyles)
            return;
        const styles = [];
        const classNames = new Set();
        for (const { className, style, atRules } of parent.childStyles) {
            const match = atRules.every(([rule, params]) => {
                return (0, match_at_rule_1.matchChildAtRule)(rule, params, options);
            });
            if (match) {
                classNames.add(className);
                styles.push(style);
            }
        }
        if (styles.length === 0) {
            return;
        }
        const className = `${[...classNames].join(" ")}.child`;
        if (this.snapshot[className])
            return this.snapshot[className];
        this.snapshot = { ...this.snapshot, [className]: styles };
        return this.snapshot[className];
    }
}
exports.StyleSheetStore = StyleSheetStore;
exports.StoreContext = (0, react_1.createContext)(new StyleSheetStore());
